/* The update is done using the cluster algorithm of Swendsen and Wang,
   Phys Rev Lett 58, 86 (1987). 
   Ants-in-the-labyrinth is used to label the clusters internal to the 
   processor, and a parallel component labeling algorithm (either local 
   label propagation or heirarchical multi-scale) is used to determine 
   the global cluster labels.
   
   Paul Coddington and Clive Baillie,  November 1989 
*/

#include "comext.h"
#include <stdio.h>

short newspin[MAXCLUSTERS/2];

SWupdate(nsweeps)
int nsweeps;
{
int i,j, isweep, n, cluster_label;
float pranf();

   for (isweep=0;isweep<nsweeps;isweep++) {

#ifdef DEBUG
printf("Updating... isweep = %d\n",isweep+1);
fflush(stdout);
#endif DEBUG

      /* Set up the bonds */
      for (i=1;i<=isiz;i++) {
      for (j=1;j<=jsiz;j++) {

	     if (spin[i][j] == spin[i+1][j]) 
	        ibond[i][j] = (pranf() < p);
             else
	        ibond[i][j] = 0;

	     if (spin[i][j] == spin[i][j+1]) 
	        jbond[i][j] = (pranf() < p);
             else
	        jbond[i][j] = 0;

      }  /* end of bond loop over jsiz */
      }  /* end of bond loop over isiz */

      /* Label the sub-clusters of bonded spins on each processor */
      clusters();

      /* Assign a random spin value to each cluster */
      for (n=0;n<Nclusters;n++) {
         newspin[n] = Q * pranf();
         if (newspin[n] == Q) {
	    newspin[n] = 0;
         }
      }

      /* Set up the global cluster labels */
      for (i=0; i<Nclusters; i++) {
	 N[i] = i+offset;
      }

      /* Pass bond edges */
      bond_edges();

#ifdef DEBUG
printf("Before global labeling...\n"); fflush(stdout);
printcluster(cluster,0);
printf("newspins...\n");
fmulti(stdout);
printf("%5d  ",offset);
for (n=0;n<Nclusters;n++) printf(" %d",newspin[n]);
printf("\n"); fflush(stdout);
fsingl(stdout);
#endif DEBUG

#ifdef GRAPHICS
      graphics();
      /* wait a bit */
      for (i=1;i<=10000;i++) pranf();
#endif GRAPHICS

      /* Get the global cluster labels by matching up the edges */
      global_label();

      /* Update the lattice */
      for (i=1;i<=isiz;i++) {
      for (j=1;j<=jsiz;j++) {
	    spin[i][j] = newspin[cluster[i][j]];
#ifdef DEBUG
	    cluster[i][j] = N[cluster[i][j]];
#endif DEBUG
      }}

#ifdef DEBUG
printf("After global labeling...\n"); fflush(stdout);
cluster_edges();
printcluster(cluster,0);
#endif DEBUG

#ifdef GRAPHICS
      graphics();
      /* wait a bit */
      for (i=1;i<=50000;i++) pranf();
#endif GRAPHICS

      /* Copy the edges */
      spin_edges();

#ifdef DEBUG
printf("Updated spins\n");
print(spin,0);
#endif DEBUG

   }  /* end of loop over sweeps */

   return 0;
}

